searchNodes=[{"doc":"Utility functions for Ecto related stuff and things. Check out the functions docs to see what is available.","ref":"EctoMorph.html","title":"EctoMorph","type":"module"},{"doc":"Same as cast_to_struct/2 , but raises if the data fails casting.","ref":"EctoMorph.html#cast_to_struct!/2","title":"EctoMorph.cast_to_struct!/2","type":"function"},{"doc":"Same as cast_to_struct/3 , but raises if the data fails casting.","ref":"EctoMorph.html#cast_to_struct!/3","title":"EctoMorph.cast_to_struct!/3","type":"function"},{"doc":"Takes some data and tries to convert it to a struct in the shape of the given schema. Casts values to the types defined by the schema dynamically using ecto changesets. Consider this: iex&gt; Jason . encode! ( %{ a : :b , c : Decimal . new ( &quot;10&quot; ) } ) |&gt; Jason . decode! %{ &quot;a&quot; =&gt; &quot;b&quot; , &quot;c&quot; =&gt; &quot;10&quot; } When we decode some JSON (e.g. from a jsonb column in the db or from a network request), the JSON gets decode d by our Jason lib, but not all of the information is preserved; any atom keys become strings, and if the value is a type that is not part of the JSON spec, it is casted to a string. This means we cannot pass that JSON data directly into a struct/2 function and expect a shiny Ecto struct back (struct!/2 will just raise, and struct/2 will silently return an empty struct) UNTIL NOW! Here we take care of casting the values in the json to the type that the given schema defines, as well as turning the string keys into (existing) atoms. (We know they will be existing atoms because they will exist in the schema definitions.) We filter out any keys that are not defined in the schema, and if the first argument is a struct, we call Map.from_struct/1 on it first. This can be useful for converting data between structs. Check out the tests for more full examples. Examples iex&gt; defmodule Test do ...&gt; use Ecto.Schema ...&gt; ...&gt; embedded_schema do ...&gt; field ( :pageviews , :integer ) ...&gt; end ...&gt; end ...&gt; { :ok , test = % Test { } } = cast_to_struct ( %{ &quot;pageviews&quot; =&gt; &quot;10&quot; } , Test ) ...&gt; test . pageviews 10 iex&gt; defmodule Test do ...&gt; use Ecto.Schema ...&gt; ...&gt; embedded_schema do ...&gt; field ( :pageviews , :integer ) ...&gt; end ...&gt; end ...&gt; json = %{ &quot;pageviews&quot; =&gt; &quot;10&quot; , &quot;ignored_field&quot; =&gt; &quot;ten&quot; } ...&gt; { :ok , test = % Test { } } = cast_to_struct ( json , Test ) ...&gt; test . pageviews 10","ref":"EctoMorph.html#cast_to_struct/2","title":"EctoMorph.cast_to_struct/2","type":"function"},{"doc":"Takes some data and tries to convert it to a struct in the shape of the given schema. Casts values to the types defined by the schema dynamically using ecto changesets. Accepts a whitelist of fields that you allow updates / inserts on. This list of fields can define fields for inner schemas also like so: EctoMorph . cast_to_struct ( json , SchemaUnderTest , [ :boolean , :name , :binary , :array_of_ints , steamed_hams : [ :pickles , double_nested_schema : [ :value ] ] ] ) We filter out any keys that are not defined in the schema, and if the first argument is a struct, we call Map.from_struct/1 on it first. This can be useful for converting data between structs.","ref":"EctoMorph.html#cast_to_struct/3","title":"EctoMorph.cast_to_struct/3","type":"function"},{"doc":"Deep filters the data to only include those fields that appear in the given schema and in any of the given schema's relations. If the schema has_one(:foo, Foo) and data has :foo as an key, then the value under :foo in data will be filtered by the fields in Foo. This will happen for all casts, embeds, and virtual fields. There is no way to determine via reflection which schema a through relation points to, so by default they are filtered by their own schema if they are a map. This is useful for converting a struct into a map, eliminating internal Ecto fields in the process. Options When filtering you can optionally choose to nillify Ecto.Association.NotLoaded structs. By default they are passed through as is, but you can nillify them like this: deep_filter_by_schema_fields ( data , MySchema , filter_not_loaded : true ) Examples iex&gt; deep_filter_by_schema_fields ( %{ a : &quot;c&quot; , ignored : true , stuff : &quot;nope&quot; } , A ) %{ a : &quot;c&quot; } iex&gt; data = %{ relation : % Ecto.Association.NotLoaded { } } ...&gt; deep_filter_by_schema_fields ( data , A , filter_not_loaded : true ) %{ relation : nil }","ref":"EctoMorph.html#deep_filter_by_schema_fields/3","title":"EctoMorph.deep_filter_by_schema_fields/3","type":"function"},{"doc":"Returns a map of all of the given schema's fields contained within data. This is not recursive so look at deep_filter_by_schema_fields if you want a recursive version. Options :filter_not_loaded - This will nillify any Ecto.Association.NotLoaded structs in the map, setting the value to be nil for any non loaded association. :filter_assocs - This will remove any key from data that is the same as any of the associations (including embeds) in schema. iex&gt; data = %{ id : 1 , other : % Ecto.Association.NotLoaded { } } ...&gt; filter_by_schema_fields ( data , MySchema , filter_not_loaded : true ) %{ id : 1 , other : nil } iex&gt; data = %{ id : 1 , other : % AnotherOne { } } ...&gt; filter_by_schema_fields ( data , MySchema ) %{ id : 1 , other : % AnotherOne { } }","ref":"EctoMorph.html#filter_by_schema_fields/3","title":"EctoMorph.filter_by_schema_fields/3","type":"function"},{"doc":"Casts the given data into a changeset according to the types defined by the given schema . It ignores any fields in data that are not defined in the schema, and recursively casts any embedded fields to a changeset also. Accepts a different struct as the first argument, calling Map.to_struct on it first. Also allows the schema to be an existing struct, in which case it will infer the schema from the struct, and effectively update that struct with the changes supplied in data. Examples ...&gt; data = %{ ...&gt; &quot;integer&quot; =&gt; &quot;77&quot; , ...&gt; &quot;steamed_hams&quot; =&gt; [ %{ ...&gt; &quot;pickles&quot; =&gt; 1 , ...&gt; &quot;sauce_ratio&quot; =&gt; &quot;0.7&quot; , ...&gt; &quot;double_nested_schema&quot; =&gt; %{ &quot;value&quot; =&gt; &quot;works!&quot; } ...&gt; } ] , ...&gt; } ...&gt; EctoMorph . generate_changeset ( data , % SchemaUnderTest { integer : 2 } ) ...&gt;","ref":"EctoMorph.html#generate_changeset/2","title":"EctoMorph.generate_changeset/2","type":"function"},{"doc":"Takes in a map of data and creates a changeset out of it by casting the data recursively, according to the whitelist of fields in fields. The map of data may be a struct, and the fields whitelist can whitelist fields of nested relations by providing a list for them as well. Examples If we provide a whitelist of fields, we will be passed a changeset for the changes on those fields only: ...&gt; data = %{ ...&gt; &quot;integer&quot; =&gt; &quot;77&quot; , ...&gt; &quot;steamed_hams&quot; =&gt; [ %{ ...&gt; &quot;pickles&quot; =&gt; 1 , ...&gt; &quot;sauce_ratio&quot; =&gt; &quot;0.7&quot; , ...&gt; &quot;double_nested_schema&quot; =&gt; %{ &quot;value&quot; =&gt; &quot;works!&quot; } ...&gt; } ] , ...&gt; } ...&gt; EctoMorph . generate_changeset ( data , SchemaUnderTest , [ :integer ] ) ...&gt; We can also define whitelists for any arbitrarily deep relation like so: ...&gt; data = %{ ...&gt; &quot;integer&quot; =&gt; &quot;77&quot; , ...&gt; &quot;steamed_hams&quot; =&gt; [ %{ ...&gt; &quot;pickles&quot; =&gt; 1 , ...&gt; &quot;sauce_ratio&quot; =&gt; &quot;0.7&quot; , ...&gt; &quot;double_nested_schema&quot; =&gt; %{ &quot;value&quot; =&gt; &quot;works!&quot; } ...&gt; } ] , ...&gt; } ...&gt; EctoMorph . generate_changeset ( data , SchemaUnderTest , [ ...&gt; :integer , ...&gt; steamed_hams : [ :pickles , double_nested_schema : [ :value ] ] ...&gt; ] )","ref":"EctoMorph.html#generate_changeset/3","title":"EctoMorph.generate_changeset/3","type":"function"},{"doc":"Essentially a wrapper around Ecto.Changeset.apply_action! where the action is create. It will create a struct out of a valid changeset and raise in the case of an invalid one.","ref":"EctoMorph.html#into_struct!/1","title":"EctoMorph.into_struct!/1","type":"function"},{"doc":"Take a changeset and returns a struct if there are no errors on the changeset. Returns an error tuple with the invalid changeset otherwise.","ref":"EctoMorph.html#into_struct/1","title":"EctoMorph.into_struct/1","type":"function"},{"doc":"Creates a map out of the Ecto struct, removing the internal ecto fields. Optionally you can remove the inserted_at and updated_at timestamp fields also by passing in :exclude_timestamps as an option This function is not deep. Prefer deep_filter_by_schema_fields or filter_by_schema_fields Examples iex&gt; map_from_struct ( % Test { } , [ :exclude_timestamps ] ) % Test { foo : &quot;bar&quot; , id : 10 } iex&gt; map_from_struct ( % Test { } ) % Test { foo : &quot;bar&quot; , updated_at : ~N[2000-01-01 23:00:07] , inserted_at : ~N[2000-01-01 23:00:07] , id : 10 } iex&gt; map_from_struct ( % Test { } , [ :exclude_timestamps , :exclude_id ] ) % Test { foo : &quot;bar&quot; }","ref":"EctoMorph.html#map_from_struct/2","title":"EctoMorph.map_from_struct/2","type":"function"},{"doc":"","ref":"EctoMorph.html#split_fields/1","title":"EctoMorph.split_fields/1","type":"function"},{"doc":"Attempts to update the given Ecto Schema struct with the given data by casting data and merging it into the struct. Uses cast and changesets to recursively update any nested relations also. Accepts a whitelist of fields for which updates can take place on. The whitelist can be arbitrarily nested, and Data may be a map, or another struct of any kind. See examples below. Examples iex&gt; MyApp.Repo . get ( Thing , 10 ) |&gt; EctoMorph . update As with cast_to_struct, the data you are updating struct you are updating can be a","ref":"EctoMorph.html#update_struct/2","title":"EctoMorph.update_struct/2","type":"function"},{"doc":"","ref":"EctoMorph.html#update_struct/3","title":"EctoMorph.update_struct/3","type":"function"},{"doc":"Allows us to specify validations for nested changesets. Accepts a path to a nested changeset, and a validation function. The validation fun will be passed the changeset at the end of the path, and the result of the validation function will be merged back into the parent changeset. If a changeset is invalid, the parent will also be marked as valid?: false (as well as any changeset between the root changeset and the nested one), but the error messages will remain on the changeset they are relevant for. This is in line with how Ecto works elsewhere like in cast_embed etc. To get the nested error messages you can use Ecto.Changeset.traverse_errors This works with *_many relations by validating the list of changesets. If you are validating their nested relations, each changeset in the list must have the nested relation in their changes. If you provide a path to changeset that does not exist in changes then no validation will run for that changeset. This is to allow partial updates - essentially only validating changes that exist. If you want to validate that a set of fields are present (either on the changeset.changes or in changeset.data), then use EctoMorph.validate_required/2 Examples EctoMorph . generate_changeset ( %{ nested : %{ foo : 3 } } ) |&gt; EctoMorph . validate_nested_changeset ( [ :nested ] , fn changeset -&gt; Ecto.Changeset . validate_number ( changeset , :foo , greater_than : 5 ) end ) changeset = EctoMorph . generate_changeset ( %{ nested : %{ double_nested : %{ field : 6 } } } ) EctoMorph . validate_nested_changeset ( changeset , [ :nested , :double_nested ] , &amp; MySchema . validate / 1 )","ref":"EctoMorph.html#validate_nested_changeset/3","title":"EctoMorph.validate_nested_changeset/3","type":"function"},{"doc":"Validates whether a changeset has a the given fields. You can pass in relations and they will be required, and you can pass in nested keys which will also be validated. For the relations, this follows the semantics of Ecto.Changeset.validate_required and will check changes for a non null relation, then check data. If either are non null the validation will pass allowing the possibility for partial updates. Examples EctoMorph . generate_changeset ( %{ my : :data , relation : %{ } } , MyModule ) |&gt; EctoMorph . validate_required ( [ :relation ] ) EctoMorph . generate_changeset ( %{ my : :data , relation : %{ nested_thing : %{ } } } , MyModule ) |&gt; EctoMorph . validate_required ( [ relation : :nested_thing ] ) data = %{ my : :data , relation : %{ nested_thing : %{ thing : 1 } } } EctoMorph . validate_required ( data , [ relation : [ nested_thing : :thing ] ] )","ref":"EctoMorph.html#validate_required/2","title":"EctoMorph.validate_required/2","type":"function"},{"doc":"validate_nested_changeset requires a path in which each location points to a nested changeset. If we are not given that, we raise an error.","ref":"EctoMorph.InvalidPathError.html","title":"EctoMorph.InvalidPathError","type":"exception"},{"doc":"","ref":"EctoMorph.InvalidValidationFunction.html","title":"EctoMorph.InvalidValidationFunction","type":"exception"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"License","type":"extras"},{"doc":"EctoMorph morphs your Ecto capabilities into the s t r a t o s p h e r e ! Parse incoming data into custom structs, then validate it. Usually you have to do something like this: defmodule Embed do use Ecto.Schema embedded_schema do field ( :bar , :string ) end end defmodule Test do use Ecto.Schema embedded_schema do field ( :thing , :string ) embeds_one ( :embed , Embed ) end Ecto.Changeset . cast ( % Test { } , %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } , [ :thing ] ) |&gt; Ecto.Changeset . cast_embed ( :embed ) Now we can do this: data = %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , Test ) # or data = %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , Test , [ :thing , embed : [ :bar ] ] ) # The data can also be a struct so this would work: data = % Test { thing : &quot;foo&quot; , embed : % Embed { bar : &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , Test , [ :thing , embed : [ :bar ] ] ) # So would this: data = %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , % Test { } , [ :thing , embed : [ :bar ] ] ) # Changes can even be a different struct, if it # has overlapping keys they will be casted as expected: defmoule OtherStruct do defstruct [ :thing , :embed ] end data = % OtherStruct { thing : &quot;foo&quot; , embed : %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , % Test { } , [ :thing , embed : [ :bar ] ] ) Or something like this: with { :ok , %{ status : 200 , body : body } } &lt;- HTTPoison . get ( &quot;mygreatapi.co.uk&quot; ) do EctoMorph . cast_to_struct ( Jason . decode! ( body ) , User ) end We can also whitelist fields to cast / update: data = %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , Test , [ :thing ] ) data = %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } EctoMorph . cast_to_struct ( data , Test , [ :thing , embed : [ :bar ] ] ) Sometimes it makes sense to update a struct we have retrieved from the database with data from our response. We can do that like so: def update ( data ) do # This will update the db struct with the data passed in, then update the db. MyRepo . get! ( MySchema , 10 ) |&gt; EctoMorph . update_struct ( data ) |&gt; MyRepo . update! ( ) end Validations Often you'll want to do some validations, that's easy: ( %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } |&gt; EctoMorph . generate_changeset ( Test , [ :thing ] ) |&gt; Ecto.Changeset . validate_required ( [ :thing ] ) |&gt; EctoMorph . into_struct ( ) ) # or ( %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } |&gt; EctoMorph . generate_changeset ( Test , [ :thing ] ) |&gt; Ecto.Changeset . validate_change ( ... ) |&gt; Repo . insert! ) Valiating Nested Changesets Easily the coolest feature, say you have nested changesets via embeds or has_one/many, you can now specify a path to a changeset and specify a validation function for the changeset(s) at the end of that path. If your path ends at a list of changesets (because your model has a has_many relation for example), each of those changesets will be validated. %{ &quot;thing&quot; =&gt; &quot;foo&quot; , &quot;embed&quot; =&gt; %{ &quot;bar&quot; =&gt; &quot;baz&quot; } } |&gt; EctoMorph . generate_changeset ( Test ) |&gt; EctoMorph . validate_nested_changeset ( [ :embed ] , &amp; MyEmbed . validate / 1 ) # or json = %{ &quot;has_many&quot; =&gt; [ %{ &quot;steamed_hams&quot; =&gt; [ %{ &quot;pickles&quot; =&gt; 1 } , %{ &quot;pickles&quot; =&gt; 2 } ] } , %{ &quot;steamed_hams&quot; =&gt; [ %{ &quot;pickles&quot; =&gt; 1 } ] } , %{ &quot;steamed_hams&quot; =&gt; [ %{ &quot;pickles&quot; =&gt; 4 } , %{ &quot;pickles&quot; =&gt; 5 } ] } ] } # Here each of the steamed_hams above will have their pickle count validated: EctoMorph . generate_changeset ( json , MySchema ) |&gt; EctoMorph . validate_nested_changeset ( [ :has_many , :steamed_hams ] , fn changeset -&gt; Ecto.Changeset . validate_number ( changeset , :pickles , greater_than : 3 ) end ) Other abilities include creating a map from an ecto struct, dropping optional fields if you decide to: EctoMorph . map_from_struct ( % Test { } ) %{ foo : &quot;bar&quot; , updated_at : ~N[2000-01-01 23:00:07] , inserted_at : ~N[2000-01-01 23:00:07] , id : 10 } EctoMorph . map_from_struct ( % Test { } , [ :exclude_timestamps ] ) %{ foo : &quot;bar&quot; , id : 10 } EctoMorph . map_from_struct ( % Test { } , [ :exclude_timestamps , :exclude_id ] ) %{ foo : &quot;bar&quot; } and being able to filter some data by the fields in the given schema: defmodule Test do use Ecto.Schema embedded_schema do field ( :random , :string ) end end EctoMorph . filter_by_schema_fields ( %{ random : &quot;data&quot; , more : &quot;fields&quot; } , Test ) %{ random : &quot;data&quot; } You can even deep filter: defmodule OtherThing do use Ecto.Schema @primary_key false embedded_schema do field ( :id , :integer ) end end defmodule Test do use Ecto.Schema embedded_schema do field ( :random , :string ) embeds_one ( :other_thing , OtherThing ) end end data = %{ random : &quot;data&quot; , more : &quot;fields&quot; , __meta__ : &quot;stuff&quot; , other_thing : %{ id : 1 , ignored : &quot;field&quot; } } EctoMorph . deep_filter_by_schema_fields ( data , Test ) %{ random : &quot;data&quot; , other_thing : %{ id : 1 } } Deep filtering will keep virtual fields, relations and through relations. That means you can use it to create a map of the struct fields without ecto metadata if you filter it by itself: data = % Test { random : &quot;data&quot; , more : &quot;fields&quot; , __meta__ : &quot;stuff&quot; , other_thing : % OtherThing { id : 1 , ignored : &quot;field&quot; } } EctoMorph . deep_filter_by_schema_fields ( data , Test ) %{ random : &quot;data&quot; , other_thing : %{ id : 1 } } Check out the docs for more examples and specifics Contributing NB Set the MIX_ENV to :docs when publishing the package. This will ensure that modules inside test/support wont get their documentation published with the library (as they are included in the :dev env). MIX_ENV=docs mix hex.publish","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"The package can be installed by adding :ecto_morph to your list of dependencies in mix.exs : def deps do [ { :ecto_morph , &quot;~&gt; 0.1.26&quot; } ] end Documentation can be generated with ExDoc and published on HexDocs . Once published, the docs can be found at https://hexdocs.pm/ecto_morph .","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"Copyright (c) 2019 Adam Lancaster Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"}]